package be.iminds.ilabt.jfed.rspec.model;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import be.iminds.ilabt.jfed.util.GeniUrn;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * ModelRspec
 */
public interface ModelRspec {
    /*
     * General info
     */

    @Nullable
    ModelRspecType getModelRspecType();

    void setParseData(@Nullable String schemaLocation, @Nullable String defaultNamespace, @Nonnull Map<String, String> namespaces);

    @Nullable
    String getSchemaLocation();

    @Nullable
    String getOrigDefaultNamespace();

    @Nonnull
    Map<String, String> getNameSpaceInfo();

    @Nonnull
    List<GeniUrn> getAllComponentManagerUrns();

    /*
     * Nodes
     */


    void addNode(@Nonnull RspecNode node);

    /**
     * Deletes a node and all interfaces attached to it. If a link becomes surplus, it is removed also.
     *
     * @param node
     */
    void deleteNode(@Nonnull RspecNode node);

    @Nonnull
    List<? extends RspecNode> getNodes();

    @Nonnull
    List<? extends RspecNode> getNodesByAuthority(@Nonnull Server server);

    @Nullable
    RspecNode getNodeByUniqueId(@Nonnull String id);

    @Nullable
    RspecNode getNodeByClientId(@Nonnull String id);

    @Nullable
    RspecNode getNodeByComponentId(@Nonnull String id);

    /*
     * Links
     */

    void addLink(@Nonnull RspecLink link);

    /**
     * Removes the link from the Rspec
     *
     * @param link
     */
    void deleteLink(@Nonnull RspecLink link);

    @Nonnull
    List<? extends RspecLink> getLinks();

    @Nullable
    RspecLink getLinkByClientId(@Nonnull String id);

    @Nullable
    RspecLink getLinkByUniqueId(@Nonnull String id);

    @Nullable
    RspecInterface getInterfaceByUniqueId(@Nonnull String uniqueId);

    @Nullable
    RspecInterface getInterfaceByClientId(@Nonnull String clientId);

    /*
     * OpenFlow DataPaths
     */

    void addOpenflowDataPath(@Nonnull OpenflowDataPath dp);

    @Nonnull
    List<? extends OpenflowDataPath> getOpenflowDataPaths();

    /*
     * Leases (NITOS-style reservations)
     */
    void addLease(@Nonnull Lease lease);

    @Nonnull
    List<? extends Lease> getLeases();

    /*
     * (Wireless) Channels
     */

    void addChannel(@Nonnull Channel channel);
    boolean deleteChannel(@Nonnull Channel channel);

    @Nonnull
    List<? extends Channel> getChannels();

    @Nullable
    Channel getChannelByComponentId(@Nonnull GeniUrn componentId);

    /*
     * Geant Testbed Service
     */

    void addGeantTestbedType(@Nonnull GeantTestbedType gtt);

    @Nonnull
    List<? extends GeantTestbedType> getGeantTestbedTypes();

    /*
     * Address Pool
     */

    void addAddressPool(@Nonnull AddressPool addressPool);
    boolean deleteAddressPool(@Nonnull AddressPool addressPool);
    @Nonnull
    List<? extends AddressPool> getAddressPools();
    AddressPool getAddressPoolByClientId(@Nonnull String clientId);


    /*
     * ExecuteAnsibleCookbook
     */

    void addExecuteAnsiblePlaybook(@Nonnull ExecuteAnsiblePlaybook executeAnsiblePlaybook);
    boolean deleteExecuteAnsiblePlaybook(@Nonnull ExecuteAnsiblePlaybook executeAnsiblePlaybook);
    @Nonnull
    List<? extends ExecuteAnsiblePlaybook> getExecuteAnsiblePlaybooks();
    /*
     * DistributeSshKeypair
     */

    void addDistributeSshKeypair(@Nonnull DistributeSshKeypair distributeSshKeypair);
    boolean deleteDistributeSshKeypair(@Nonnull DistributeSshKeypair distributeSshKeypair);
    @Nonnull
    List<? extends DistributeSshKeypair> getDistributeSshKeypairs();


    @Nonnull
    List<UserSpec> getSshKeys();

    void setSshKeys(@Nonnull List<UserSpec> newKeys);

    @Nonnull
    String nextLinkName();

    @Nonnull
    String nextNodeName();

    @Nullable
    String getType();

    @Nonnull
    String nextIfaceName(@Nonnull RspecNode node);


    @Nonnull
    List<ExtraXml> getExtraXml();
    void setExtraXml(@Nonnull List<ExtraXml> extraXml);

    /**
     * For the moment, this does nothing more than call toGeni3RequestRspec_internal and write Xml To string.
     * Later, this can be edited to handle special cases in the geni request rspec v3 generation.
     * <p/>
     * only the PLE case is handled now, but it works automatically, without a need to specify it.
     */
    @Nonnull
    String toGeni3Rspec(@Nonnull RequestRspecSpecialCases specialCase);
    @Nonnull
    String toGeni3Rspec();

    @Nullable
    Date getExpires() throws ParseException;

    void setExpires(@Nullable Date expires);

    @Nullable
    String getExpiresString();

    void setExpires(@Nullable String expires);

    /* return or set to null null means tag not present in rspec xml */
    @Nullable
    Integer getRoutableAddressesAvailable();

    @Nullable
    Integer getRoutableAddressesConfigured();

    void setRoutableAddressesAvailable(@Nullable Integer i);

    void setRoutableAddressesConfigured(@Nullable Integer i);

    @Nullable
    GeantTestbedType getGeantTestbedTypeByName(@Nonnull String geantTestbedTypeName);

    @Nullable
    Integer getMultiplexFactor();
    void setMultiplexFactor(@Nullable Integer multiplexFactor);

    enum RequestRspecSpecialCases {NONE, PLE, NITOS, NITOS_NO_E_NODE_B_HACK }

    @Nonnull
    ModelRspec copy();
}
